#pragma mark ----------

#include <PalmOS.h>
#include "FEPTglCS.h"
#include "FEPTglCS35to41rsrc.h"

typedef struct 
	{
	UInt32 On:1;
	UInt32 Off:1;
	UInt32 NichiEiF:1;
	UInt32 HiraKataF:1;
	UInt32 HenkanF:1;
	UInt32 AutoEnbleF:1;
	UInt32 BlcAtShftF:1;
	UInt32 DblBckLgtF:1;
	UInt32 CmdBarIsActive:1;
	UInt32 UnUsed:23;
	} FEPTglPreferenceType;

#define IsOn		1
#define IsOff		0
#define IsActive	1
#define IsDeactive	0

void HwrDisplayAttributes(Boolean set, UInt8 kind, void *val)
	SYS_TRAP(sysTrapHwrDisplayAttributes);

#define appFileCreator			'FTgC'
#define appVersionNum			0x03
#define appPrefID				0x01
#define appPrefVersionNum		0x03

#define appLocalID				0x00	//Ftr
#define appStatus				0x01	//Ftr
#define cmdbarRscdbP			0x02	//Ftr

#define appHackTrapAddress		0x10	//Ftr

typedef Err (*EvtGetEventTrapType)
	(EventType *event, Int32 timeout);

#pragma mark ----------

static void DeactivateTrapHack(void)
{
	UInt32 trapadd;

	if (FtrGet (appFileCreator, appHackTrapAddress, &trapadd) == 0)
	{
		SysSetTrapAddress (sysTrapEvtGetEvent, (UInt32*)trapadd);
		FtrUnregister (appFileCreator, appHackTrapAddress);
	}
}

static Err EvtGetEventTrapHack(EventType *event, Int32 timeout)
{
	UInt32 oldtrapadd;
	EvtGetEventTrapType oldTrap;

	FtrGet (appFileCreator, appHackTrapAddress, &oldtrapadd);

	oldTrap = (EvtGetEventTrapType)oldtrapadd;

	Err error = oldTrap(event, timeout);

	SysNotifyParamType notify;
	notify.notifyType = sysNotifyEventDequeuedEvent;
	notify.broadcaster = 'psys';
	notify.notifyDetailsP = event;
	notify.userDataP = 0;
	notify.handled = false;
	notify.reserved2 = 0;

	SysNotifyBroadcast(&notify);

	return error;
}

static void ActivateTrapHack(void)
{
	UInt32 trapadd;
	if (FtrGet (appFileCreator, appHackTrapAddress, &trapadd) != 0)
	{
		trapadd = (UInt32)SysGetTrapAddress (sysTrapEvtGetEvent);
		FtrSet (appFileCreator, appHackTrapAddress, (UInt32)SysGetTrapAddress (sysTrapEvtGetEvent));
		SysSetTrapAddress (sysTrapEvtGetEvent, &EvtGetEventTrapHack);
	}
}

#pragma mark ----------

static void Deactivate(void)
{
	LocalID dbID;
		FtrGet (appFileCreator, appLocalID, &dbID);
	FEPTglPreferenceType prefs;
		FtrGet (appFileCreator, appStatus, (UInt32*)&prefs);
		prefs.CmdBarIsActive = IsDeactive;
		FtrSet (appFileCreator, appStatus , *(UInt32*)&prefs);

	if (prefs.BlcAtShftF == IsOff)
	{
		SysNotifyUnregister(0, dbID, sysNotifyEventDequeuedEvent, sysNotifyNormalPriority);
		DeactivateTrapHack();
	}

	AlmSetAlarm (0, dbID, NULL, 0, NULL);
	MenuEraseStatus (NULL);
}

static void Activate(void)
{
	LocalID dbID;
		FtrGet (appFileCreator, appLocalID, &dbID);
	FEPTglPreferenceType prefs;
		FtrGet (appFileCreator, appStatus, (UInt32*)&prefs);
		prefs.CmdBarIsActive = IsActive;
		FtrSet (appFileCreator, appStatus , *(UInt32*)&prefs);

	SysNotifyRegister (0, dbID, sysNotifyEventDequeuedEvent, NULL, sysNotifyNormalPriority, NULL);
	ActivateTrapHack();

	AlmSetAlarm (0, dbID, NULL, 0, NULL);
	AlmSetAlarm (0, dbID, NULL, (TimGetSeconds () + 2), NULL);

	switch (FtrGet (appFileCreator, cmdbarRscdbP, &dbID))
	{
	case ftrErrNoSuchFeature:
		dbID = DmFindDatabase (0, "FEPTglCS-Ico");
		if (dbID != 0)
		{
			DmOpenRef dbP = DmOpenDatabase (0, dbID, dmModeLeaveOpen);
				FtrSet (appFileCreator, cmdbarRscdbP, (UInt32)dbP);
		}	else FtrUnregister (appFileCreator, cmdbarRscdbP);

	default:
		if (prefs.NichiEiF == IsOn)
			MenuCmdBarAddButton (menuCmdBarOnRight,
				NichiEiBitmapFamily, menuCmdBarResultChar,
				vchrTsm4, "p");
		if (prefs.HiraKataF == IsOn)
			MenuCmdBarAddButton (menuCmdBarOnRight,
				HiraKataBitmapFamily, menuCmdBarResultChar,
				vchrTsm3, "A");
		if (prefs.HenkanF == IsOn)
			MenuCmdBarAddButton (menuCmdBarOnRight,
				HenkanBitmapFamily, menuCmdBarResultChar,
				vchrTsm1, "ϊ");
	}
}

#pragma mark ----------

static Boolean ChkBacklight(void)
{
	Boolean val;
	HwrDisplayAttributes(false, 0x12, &val);
	return val;
}

static void ChkAutoShift(void)
{
	FormType *frmP = FrmGetActiveForm ();
	FieldType* fldP;
	FieldAttrType attr;
	Boolean capsLock;
	Boolean numLock;
	UInt16 tempShift;
	Boolean autoShifted;

	if (frmP != NULL)
	{
		UInt16 objIndex = FrmGetFocus (frmP);
		if (objIndex != noFocus)
		{
			if (FrmGetObjectType (frmP, objIndex) == frmFieldObj)
			{
				fldP = (FieldType*)FrmGetObjectPtr (frmP, objIndex);
				goto checkAutoshifting;
			}
			if (FrmGetObjectType (frmP, objIndex) == frmTableObj)
			{
				TableType *tableP;
				tableP = (TableType*)FrmGetObjectPtr (frmP, objIndex);
				fldP = (FieldType*)TblGetCurrentField (tableP);
				if (fldP != NULL)
					goto checkAutoshifting;
			}
		}
	}
return;

checkAutoshifting:
	FldGetAttributes (fldP, &attr);
	if (attr.autoShift == IsOn)
	{
//		char rr[20];
//		StrIToH (rr, (UInt32)fldP);
//		WinDrawChars (rr+4, 4, 0, 0);

		attr.autoShift = IsOff;
		FldSetAttributes (fldP, &attr);
		GrfGetState (&capsLock, &numLock, &tempShift, &autoShifted);
		if ((tempShift == grfTempShiftUpper) && (capsLock != true))
		{
			GrfSetState (capsLock, numLock, false);
		}
	}
}

#pragma mark ----------

static void NtfyHandle(EventPtr DequedEvent)
{
	FEPTglPreferenceType prefs;
		FtrGet (appFileCreator, appStatus, (UInt32*)&prefs);

/*	if (DequedEvent->eType == 0x0100)		//penDownEvent 0x0001
	{
		Coord extentX;
		Coord extentY;
		WinGetDisplayExtent (&extentX, &extentY);

		RectangleType rP;
		WinGetWindowFrameRect (WinGetActiveWindow (), &rP);
		if ((Int16)Swap16(DequedEvent->screenY) <= (Int16)(extentY - rP.topLeft.y - 20))
			Deactivate();
	}*/

	if (prefs.CmdBarIsActive == IsDeactive)
	{
		switch (/*Swap16*/(DequedEvent->eType))
		{
		case nilEvent:
		case penUpEvent:
		case keyDownEvent:
		case winExitEvent:
		case fldHeightChangedEvent:
		case tblSelectEvent:
			if (prefs.BlcAtShftF == IsOn)
			{
				ChkAutoShift();
			}
		default:
			break;
		}
	}

	if (prefs.CmdBarIsActive == IsActive)
	{
		switch (/*Swap16*/(DequedEvent->eType))
		{
		case keyDownEvent:
			switch (/*Swap16*/(DequedEvent->data.keyDown.chr))
			{
			case vchrLaunch:
			case vchrMenu:
			case vchrKeyboard:
			case vchrFind:
			case vchrCalc:
			case vchrKeyboardAlpha:
			case vchrKeyboardNumeric:
			case vchrTsm1:
			case vchrTsm2:
			case vchrTsm3:
			case vchrTsm4:
			case vchrBrightness:
			case vchrContrast:
				Deactivate();
				break;

			case chrSpace:	//chrSpace 0x0020
				DequedEvent->data.keyDown.chr = vchrTsm1;	//vchrTsm1 0x0118 Henkan
				DequedEvent->data.keyDown.modifiers = 0x0008;
				Deactivate();
				break;

			case chrLineFeed:	//chrLineFeed 0x000a
				DequedEvent->data.keyDown.chr = vchrTsm3;	//vchrTsm3 0x011a HiraKata
				DequedEvent->data.keyDown.modifiers = 0x0008;
				Deactivate();
				break;

			case vchrCommand:	//vchrCommand 0x0106
				DequedEvent->data.keyDown.chr = vchrTsm4;	//vchrTsm4 0x011b Nichiei
				DequedEvent->data.keyDown.modifiers = 0x0008;
				Deactivate();
				break;

			case chrBackspace:	//chrBackspace 0x0008
				DequedEvent->data.keyDown.chr = vchrLaunch;	//vchrLaunch 0x0108 Launch
				DequedEvent->data.keyDown.modifiers = 0x0008;
				Deactivate();
				break;
			}
			break;

		case menuEvent:
			Deactivate();
			break;
		}
	}
} 

#pragma mark ----------

static void SetFEPTglPreference(FEPTglPreferenceType *prefs)
{
	PrefSetAppPreferences (appFileCreator, appPrefID, appPrefVersionNum, 
		prefs, sizeof (FEPTglPreferenceType), false);
}

static void GetFEPTglPreference(FEPTglPreferenceType *prefs)
{
	UInt16 prefsSize;

	prefsSize = sizeof(FEPTglPreferenceType);
	if (PrefGetAppPreferences(appFileCreator, appPrefID, prefs, &prefsSize, false) == 
		noPreferenceFound)
		{
			prefs->On         = IsOff;
			prefs->Off        = IsOn;
			prefs->NichiEiF   = IsOff;
			prefs->HiraKataF  = IsOff;
			prefs->HenkanF    = IsOff;
			prefs->AutoEnbleF = IsOff;
			prefs->BlcAtShftF = IsOff;
			prefs->DblBckLgtF = IsOff;
		}
	prefs->CmdBarIsActive = IsDeactive;
	prefs->UnUsed = 0;
	SetFEPTglPreference(prefs);
}

#pragma mark ----------

static void Disable(void)
{
	UInt16 cardNo;
	LocalID dbID;
	SysCurAppDatabase (&cardNo, &dbID);

	SysNotifyUnregister(cardNo, dbID, sysNotifyMenuCmdBarOpenEvent, sysNotifyNormalPriority);
	SysNotifyUnregister(cardNo, dbID, sysNotifyEventDequeuedEvent, sysNotifyNormalPriority);
	DeactivateTrapHack();

	FtrUnregister (appFileCreator, appLocalID);
	FtrUnregister (appFileCreator, appStatus );

	UInt32 dbP;
	if (FtrGet (appFileCreator, cmdbarRscdbP, &dbP) == 0)
	{
		if (dbP != 0)
		{
			DmCloseDatabase ((DmOpenRef)dbP);
		}
	}

	FtrUnregister (appFileCreator, cmdbarRscdbP);
}

static void Enable(void)
{
	UInt16 cardNo;
	LocalID dbID;
	SysCurAppDatabase (&cardNo, &dbID);

	SysNotifyRegister (cardNo, dbID, sysNotifyMenuCmdBarOpenEvent, NULL, sysNotifyNormalPriority, NULL);

	FEPTglPreferenceType prefs;
		GetFEPTglPreference(&prefs);
	FtrSet (appFileCreator, appLocalID, dbID );
	FtrSet (appFileCreator, appStatus , *(UInt32*)&prefs);

	if (prefs.BlcAtShftF == IsOn)
	{
		SysNotifyRegister (cardNo, dbID, sysNotifyEventDequeuedEvent, NULL, sysNotifyNormalPriority, NULL);
		ActivateTrapHack();
	}
}

#pragma mark ----------

static void GetCtrlValue(FormPtr frmP)
{
	FEPTglPreferenceType prefs;

	if ( FrmGetControlValue (frmP, FrmGetObjectIndex (frmP, MainONPushButton)) == true)
		prefs.On = IsOn; else prefs.On = IsOff;
	if ( FrmGetControlValue (frmP, FrmGetObjectIndex (frmP, MainOFFPushButton)) == true)
		prefs.Off = IsOn; else prefs.Off = IsOff;
	if ( FrmGetControlValue (frmP, FrmGetObjectIndex (frmP, MainNichiEiCheckbox  )) == true)
		prefs.NichiEiF = IsOn; else prefs.NichiEiF = IsOff;
	if ( FrmGetControlValue (frmP, FrmGetObjectIndex (frmP, MainHiraKataCheckbox )) == true)
		prefs.HiraKataF = IsOn; else prefs.HiraKataF = IsOff;
	if ( FrmGetControlValue (frmP, FrmGetObjectIndex (frmP, MainHenkanCheckbox   )) == true)
		prefs.HenkanF = IsOn; else prefs.HenkanF = IsOff;
	if ( FrmGetControlValue (frmP, FrmGetObjectIndex (frmP, MainEnblRstCheckbox)) == true)
		prefs.AutoEnbleF = IsOn; else prefs.AutoEnbleF = IsOff;
	if ( FrmGetControlValue (frmP, FrmGetObjectIndex (frmP, MainBlcAtShftCheckbox)) == true)
		prefs.BlcAtShftF = IsOn; else prefs.BlcAtShftF = IsOff;
	if ( FrmGetControlValue (frmP, FrmGetObjectIndex (frmP, MainDblBcklgtCheckbox)) == true)
		prefs.DblBckLgtF = IsOn; else prefs.DblBckLgtF = IsOff;
	SetFEPTglPreference(&prefs);
	
	if (prefs.On == IsOn)
	{
		Enable();
	}
	else
	{
		Disable();
	}
}

static void SetCtrlValue(FormPtr frmP)
{
	FEPTglPreferenceType prefs;
	GetFEPTglPreference(&prefs);

	FrmSetControlValue (frmP, FrmGetObjectIndex (frmP, MainONPushButton     ), prefs.On        );
	FrmSetControlValue (frmP, FrmGetObjectIndex (frmP, MainOFFPushButton    ), prefs.Off       );
	FrmSetControlValue (frmP, FrmGetObjectIndex (frmP, MainNichiEiCheckbox  ), prefs.NichiEiF  );
	FrmSetControlValue (frmP, FrmGetObjectIndex (frmP, MainHiraKataCheckbox ), prefs.HiraKataF );
	FrmSetControlValue (frmP, FrmGetObjectIndex (frmP, MainHenkanCheckbox   ), prefs.HenkanF   );
	FrmSetControlValue (frmP, FrmGetObjectIndex (frmP, MainEnblRstCheckbox  ), prefs.AutoEnbleF);
	FrmSetControlValue (frmP, FrmGetObjectIndex (frmP, MainBlcAtShftCheckbox), prefs.BlcAtShftF);
	FrmSetControlValue (frmP, FrmGetObjectIndex (frmP, MainDblBcklgtCheckbox), prefs.DblBckLgtF);

	if (DmFindDatabase (0, "FEPTglCS-Ico") != 0)
	{
		FrmShowObject (frmP, FrmGetObjectIndex (frmP, MainNichiEiCheckbox ));
		FrmShowObject (frmP, FrmGetObjectIndex (frmP, MainHiraKataCheckbox));
		FrmShowObject (frmP, FrmGetObjectIndex (frmP, MainHenkanCheckbox  ));
	}
}

#pragma mark ----------

static UInt32 StarterPalmMain(UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
{
//	Err error;

	switch (cmd)
		{
		case sysAppLaunchCmdNotify:
			SysNotifyParamType* NtfyPrm = (SysNotifyParamType*)cmdPBP;
			switch (NtfyPrm->notifyType)
				{
				case sysNotifyEventDequeuedEvent:
					EventPtr DequedEvent;
					DequedEvent = (EventType*)NtfyPrm->notifyDetailsP;
					NtfyHandle (DequedEvent);
					break;

				case sysNotifyMenuCmdBarOpenEvent:
					FEPTglPreferenceType prefs;
					FtrGet (appFileCreator, appStatus, (UInt32*)&prefs);
					if (prefs.DblBckLgtF == IsOn)
						if (ChkBacklight() == false)
							break;
					Activate();
					break;
				}
			break;

		case sysAppLaunchCmdAlarmTriggered:
			Deactivate ();
			break;

		case sysAppLaunchCmdNormalLaunch:
			FormPtr frmP = FrmInitForm (MainForm);		//Initialize dialogue form
			SetCtrlValue(frmP);				 			//Set control value
			if (FrmDoDialog (frmP) == MainDoneButton)	//Draw dialogue form
				GetCtrlValue(frmP);						//Get control value
			break;

		case sysAppLaunchCmdSystemReset:
			FEPTglPreferenceType prefs;
			GetFEPTglPreference(&prefs);
			if (prefs.On == IsOn)
			{
				if (prefs.AutoEnbleF == IsOn)
					Enable();						//activate
				else
				{
					prefs.On  = IsOff;
					prefs.Off = IsOn;
					SetFEPTglPreference(&prefs);
				}
			}
			break;

		default:
			break;
		}
	return errNone;
}

UInt32 PilotMain( UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
{
	return StarterPalmMain(cmd, cmdPBP, launchFlags);
}
