#include <PalmOS.h>
#include "FEPTglPS.h"

typedef struct 
	{
	UInt16 On;
	UInt16 Off;
	UInt16 NichiEiF;
	UInt16 HiraKataF;
	UInt16 HenkanF;
	UInt16 AutoEnbleF;
	UInt16 DblBckLgtF;
	} FEPTglPreferenceType;

void HwrDisplayAttributes(Boolean set, UInt8 kind, void *val)
	SYS_TRAP(sysTrapHwrDisplayAttributes);

#define appFileCreator			'FTgP'
#define appVersionNum			0x02
#define appPrefID				0x01
#define appPrefVersionNum		0x01

#define appLocalID				0x00	//Ftr
#define psHiraKata				0x02	//Ftr
//#define psHenkan				0x03	//Ftr
#define chkBacklight			0x04	//Ftr
#define LnchSilkBtnTopLeft		0x06	//Ftr
#define MenuSilkBtnTopLeft		0x07	//Ftr
#define SilkBtnExtent			0x08	//Ftr

//#define Swap16(n) (((((UInt16)n) << 8) & 0xFF00) | ((((UInt16)n) >> 8) & 0x00FF))

static Boolean ChkBacklight(void)
{
	Boolean val;
	HwrDisplayAttributes(false, 0x12, &val);
	return val;
}

static void HandlePenStroke(SysNotifyPenStrokeType* PS)
{
	UInt32 LnchBtnTopLeft;
	UInt32 MenuBtnTopLeft;
	UInt32 BtnExtent;

	FtrGet (appFileCreator, LnchSilkBtnTopLeft, &LnchBtnTopLeft);
	FtrGet (appFileCreator, MenuSilkBtnTopLeft, &MenuBtnTopLeft);
	FtrGet (appFileCreator, SilkBtnExtent     , &BtnExtent     );

	UInt16 LnchBtnTop = (LnchBtnTopLeft & 0x0000FFFF);
	UInt16 LnchBtnLft = (LnchBtnTopLeft & 0xFFFF0000) >> 16;
	UInt16 MenuBtnTop = (MenuBtnTopLeft & 0x0000FFFF);
	UInt16 MenuBtnLft = (MenuBtnTopLeft & 0xFFFF0000) >> 16;
	UInt16 BtnExtentx = (BtnExtent & 0xFFFF0000) >> 16;
	UInt16 BtnExtenty = (BtnExtent & 0x0000FFFF);

	PointType StrtPt = PS->startPt;
	PointType EndPt = PS->endPt;

	if ((((Int16)EndPt.x - (Int16)StrtPt.x) > ((Int16)BtnExtentx / 2)) &&
		(((Int16)StrtPt.y - (Int16)EndPt.y) > ((Int16)BtnExtenty / 2)) )
	{
	//case 1 Home
		if (
			((LnchBtnTop <= StrtPt.y) && (StrtPt.y < LnchBtnTop + BtnExtenty)) &&
			((LnchBtnLft <= StrtPt.x) && (StrtPt.x < LnchBtnLft + BtnExtentx)))
		{
			EvtEnqueueKey (vchrTsm4, 0, 0);
		}

	//case 2 Menu
		if (
			((MenuBtnTop <= StrtPt.y) && (StrtPt.y < MenuBtnTop + BtnExtenty)) &&
			((MenuBtnLft <= StrtPt.x) && (StrtPt.x < MenuBtnLft + BtnExtentx)))
		{
			UInt32 Flug;
			FtrGet (appFileCreator, psHiraKata, &Flug);
			if (Flug != 0)
				EvtEnqueueKey (vchrTsm3, 0, 8);
			else
				EvtEnqueueKey (vchrTsm1, 0, 8);
		}
	}
}

static void SetFEPTglPreference(FEPTglPreferenceType *prefs)
{
	PrefSetAppPreferences (appFileCreator, appPrefID, appPrefVersionNum, 
		prefs, sizeof (FEPTglPreferenceType), false);
}

static void GetFEPTglPreference(FEPTglPreferenceType *prefs)
{
	UInt16 prefsSize;

	prefsSize = sizeof(FEPTglPreferenceType);
	if (PrefGetAppPreferences(appFileCreator, appPrefID, prefs, &prefsSize, false) == 
		noPreferenceFound)
		{
			prefs->On         = 0;
			prefs->Off        = 1;
			prefs->AutoEnbleF = 0;
			prefs->DblBckLgtF = 0;
			prefs->NichiEiF   = 0;
			prefs->HiraKataF  = 1;
			prefs->HenkanF    = 0;

			SetFEPTglPreference(prefs);
		}
}

static void Disable(void)
{
	UInt16 cardNo;
	LocalID dbID;
	SysCurAppDatabase (&cardNo, &dbID);
	SysNotifyUnregister(cardNo, dbID, sysNotifyProcessPenStrokeEvent, sysNotifyNormalPriority);

	FtrUnregister (appFileCreator, psHiraKata    );
//	FtrUnregister (appFileCreator, psHenkan      );
	FtrUnregister (appFileCreator, chkBacklight  );

	FtrUnregister (appFileCreator, LnchSilkBtnTopLeft);
	FtrUnregister (appFileCreator, MenuSilkBtnTopLeft);
	FtrUnregister (appFileCreator, SilkBtnExtent     );
}

static void Enable(void)
{
	UInt16 cardNo;
	LocalID dbID;
	SysCurAppDatabase (&cardNo, &dbID);
	SysNotifyRegister (cardNo, dbID, sysNotifyProcessPenStrokeEvent, NULL, sysNotifyNormalPriority, NULL);

	FEPTglPreferenceType prefs;
	GetFEPTglPreference(&prefs);
	
	UInt8 i;
	PenBtnInfoPtr PenBtnInfoP;
	UInt16 numButtons;
	RectangleType LnchBtnboundsR;
	RectangleType MenuBtnboundsR;
	UInt32 LnchBtnTopLeft;
	UInt32 MenuBtnTopLeft;
	UInt32 BtnExtent;

	PenBtnInfoP = (PenBtnInfoPtr)EvtGetPenBtnList(&numButtons);

	for (i = 0; i < numButtons; i++)
	{
		if ((&PenBtnInfoP[i])->asciiCode == vchrLaunch)
		{
			LnchBtnboundsR = (&PenBtnInfoP[i])->boundsR;
			LnchBtnTopLeft =
				((((UInt32)LnchBtnboundsR.topLeft.x) << 16) & 0xFFFF0000) |
					((UInt32)LnchBtnboundsR.topLeft.y);
			BtnExtent      =
				((((UInt32)LnchBtnboundsR.extent.x ) << 16) & 0xFFFF0000) |
					((UInt32)LnchBtnboundsR.extent.y);
		}
		if ((&PenBtnInfoP[i])->asciiCode == vchrMenu)
		{
			MenuBtnboundsR = (&PenBtnInfoP[i])->boundsR;
			MenuBtnTopLeft = 
				((((UInt32)MenuBtnboundsR.topLeft.x) << 16) & 0xFFFF0000) |
					((UInt32)MenuBtnboundsR.topLeft.y);
		}
	}

	FtrSet (appFileCreator, psHiraKata  , prefs.HiraKataF );
//	FtrSet (appFileCreator, psHenkan    , prefs.HenkanF   );
	FtrSet (appFileCreator, chkBacklight, prefs.DblBckLgtF);

	FtrSet (appFileCreator, LnchSilkBtnTopLeft, LnchBtnTopLeft);
	FtrSet (appFileCreator, MenuSilkBtnTopLeft, MenuBtnTopLeft);
	FtrSet (appFileCreator, SilkBtnExtent     , BtnExtent     );
}

static void GetCtrlValue(FormPtr frmP)
{
	FEPTglPreferenceType prefs;

	prefs.On         = FrmGetControlValue (frmP, FrmGetObjectIndex (frmP, MainONPushButton     ));
																		// !0:ON 0:OFF
	prefs.Off        = FrmGetControlValue (frmP, FrmGetObjectIndex (frmP, MainOFFPushButton    ));
																		// !0:ON 0:OFF
	prefs.AutoEnbleF = FrmGetControlValue (frmP, FrmGetObjectIndex (frmP, MainEnblRstCheckbox  ));
																		// !0:ON 0:OFF
	prefs.DblBckLgtF = FrmGetControlValue (frmP, FrmGetObjectIndex (frmP, MainDblBcklgtCheckbox));
																		// !0:ON 0:OFF
//	prefs.NichiEiF   = FrmGetControlValue (frmP, FrmGetObjectIndex (frmP, MainNichiEiCheckbox  ));
																		// !0:ON 0:OFF
	prefs.HiraKataF  = FrmGetControlValue (frmP, FrmGetObjectIndex (frmP, MainHiraKataCheckbox ));
																		// !0:ON 0:OFF
	prefs.HenkanF    = FrmGetControlValue (frmP, FrmGetObjectIndex (frmP, MainHenkanCheckbox   ));
																		// !0:ON 0:OFF
	SetFEPTglPreference(&prefs);
	
	if (prefs.On != 0)
	{
		Enable();
	}
	else
	{
		Disable();
	}
}

static void SetCtrlValue(FormPtr frmP)
{
	FEPTglPreferenceType prefs;
	GetFEPTglPreference(&prefs);

	FrmSetControlValue (frmP, FrmGetObjectIndex (frmP, MainONPushButton     ), prefs.On        );
	FrmSetControlValue (frmP, FrmGetObjectIndex (frmP, MainOFFPushButton    ), prefs.Off       );
//	FrmSetControlValue (frmP, FrmGetObjectIndex (frmP, MainNichiEiCheckbox  ), prefs.NichiEiF  );
	FrmSetControlValue (frmP, FrmGetObjectIndex (frmP, MainHiraKataCheckbox ), prefs.HiraKataF );
	FrmSetControlValue (frmP, FrmGetObjectIndex (frmP, MainHenkanCheckbox   ), prefs.HenkanF   );
	FrmSetControlValue (frmP, FrmGetObjectIndex (frmP, MainEnblRstCheckbox  ), prefs.AutoEnbleF);
	FrmSetControlValue (frmP, FrmGetObjectIndex (frmP, MainDblBcklgtCheckbox), prefs.DblBckLgtF);
}

static UInt32 StarterPalmMain(UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
{
//	Err error;

	switch (cmd)
		{
		case sysAppLaunchCmdNotify:
			SysNotifyParamType* NtfyPrm = (SysNotifyParamType*)cmdPBP;
			switch (NtfyPrm->notifyType)
				{
				case sysNotifyProcessPenStrokeEvent://////////////////////////////////////
					UInt32 DblBckLgtF;
					FtrGet (appFileCreator, chkBacklight, &DblBckLgtF);
					if (DblBckLgtF != 0)
						if (ChkBacklight() == false)
							break;
					SysNotifyPenStrokeType* PS = (SysNotifyPenStrokeType*)NtfyPrm->notifyDetailsP;
					HandlePenStroke (PS);
					break;
				}
			break;

		case sysAppLaunchCmdNormalLaunch:
			FormPtr frmP = FrmInitForm (MainForm);		//Initialize dialogue form
			SetCtrlValue (frmP);			 			//Set control value
			if (FrmDoDialog (frmP) == MainDoneButton)	//Draw dialogue form
				GetCtrlValue(frmP);						//Get control value
			break;

		case sysAppLaunchCmdSystemReset:
			FEPTglPreferenceType prefs;				//Check Auto Enable
			GetFEPTglPreference(&prefs);
			if (prefs.On != 0)
			{
				if (prefs.AutoEnbleF != 0)
					Enable();						//activate
				else
				{
					prefs.On  = 0;
					prefs.Off = 1;
					SetFEPTglPreference(&prefs);
				}
			}
			break;

		default:
			break;

		}
	
	return errNone;
}

UInt32 PilotMain( UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
{
	return StarterPalmMain(cmd, cmdPBP, launchFlags);
}
