#pragma mark ----------

#include <PalmOS.h>
#include "FEPTglTC.h"

typedef struct 
	{
	UInt32 On:1;
	UInt32 Off:1;
	UInt32 AutoEnbleF:1;
	UInt32 BlcAtShftF:1;
	UInt32 CmdBarIsActive:1;
	UInt32 UnUsed:27;
	} FEPTglPreferenceType;

#define IsOn		1
#define IsOff		0
#define IsActive	1
#define IsDeactive	0

void HwrDisplayAttributes(Boolean set, UInt8 kind, void *val)
	SYS_TRAP(sysTrapHwrDisplayAttributes);

#define appFileCreator			'FTgT'
#define appVersionNum			0x02
#define appPrefID				0x01
#define appPrefVersionNum		0x02

#define appLocalID				0x00	//Ftr
#define appStatus				0x01	//Ftr

#define Swap16(n) (((((UInt16)n) << 8) & 0xFF00) | ((((UInt16)n) >> 8) & 0x00FF))

#pragma mark ----------

static void Deactivate(void)
{
	LocalID dbID;
		FtrGet (appFileCreator, appLocalID, &dbID);
	FEPTglPreferenceType prefs;
		FtrGet (appFileCreator, appStatus, (UInt32*)&prefs);
		prefs.CmdBarIsActive = IsDeactive;
		FtrSet (appFileCreator, appStatus , *(UInt32*)&prefs);

	if (prefs.BlcAtShftF == IsOff)
		SysNotifyUnregister(0, dbID, sysNotifyEventDequeuedEvent, sysNotifyNormalPriority);

	AlmSetAlarm (0, dbID, NULL, 0, NULL);
	MenuEraseStatus (NULL);
}

static void Activate(void)
{
	LocalID dbID;
		FtrGet (appFileCreator, appLocalID, &dbID);
	FEPTglPreferenceType prefs;
		FtrGet (appFileCreator, appStatus, (UInt32*)&prefs);
		prefs.CmdBarIsActive = IsActive;
		FtrSet (appFileCreator, appStatus , *(UInt32*)&prefs);

	SysNotifyRegister (0, dbID, sysNotifyEventDequeuedEvent, NULL, sysNotifyNormalPriority, NULL);

	AlmSetAlarm (0, dbID, NULL, 0, NULL);
	AlmSetAlarm (0, dbID, NULL, (TimGetSeconds () + 2), NULL);
}

#pragma mark ----------

static Boolean ChkBacklight(void)
{
	Boolean val;
	HwrDisplayAttributes(false, 0x12, &val);
	return val;
}

static void ChkAutoShift(void)
{
	FormType *frmP = FrmGetActiveForm ();
	FieldType* fldP;
	FieldAttrType attr;
	Boolean capsLock;
	Boolean numLock;
	UInt16 tempShift;
	Boolean autoShifted;

	if (frmP != NULL)
	{
		UInt16 objIndex = FrmGetFocus (frmP);
		if (objIndex != noFocus)
		{
			if (FrmGetObjectType (frmP, objIndex) == frmFieldObj)
			{
				fldP = (FieldType*)FrmGetObjectPtr (frmP, objIndex);
				goto checkAutoshifting;
			}
			if (FrmGetObjectType (frmP, objIndex) == frmTableObj)
			{
				TableType *tableP;
				tableP = (TableType*)FrmGetObjectPtr (frmP, objIndex);
				fldP = (FieldType*)TblGetCurrentField (tableP);
				if (fldP != NULL)
					goto checkAutoshifting;
			}
		}
	}
return;

checkAutoshifting:
	FldGetAttributes (fldP, &attr);
	if (attr.autoShift == IsOn)
	{
//		char rr[20];
//		StrIToH (rr, (UInt32)fldP);
//		WinDrawChars (rr+4, 4, 0, 0);

		attr.autoShift = IsOff;
		FldSetAttributes (fldP, &attr);
		GrfGetState (&capsLock, &numLock, &tempShift, &autoShifted);
		if ((tempShift == grfTempShiftUpper) && (capsLock != true))
		{
			GrfSetState (capsLock, numLock, false);
		}
	}
}

#pragma mark ----------

static void NtfyHandle(EventPtr DequedEvent)
{
	FEPTglPreferenceType prefs;
		FtrGet (appFileCreator, appStatus, (UInt32*)&prefs);

	if (prefs.CmdBarIsActive == IsDeactive)
	{
		switch (Swap16(DequedEvent->eType))
		{
		case nilEvent:
		case penUpEvent:
		case keyDownEvent:
		case winExitEvent:
		case fldHeightChangedEvent:
		case tblSelectEvent:
			if (prefs.BlcAtShftF == IsOn)
			{
				ChkAutoShift();
			}
		default:
			break;
		}
	}

	if (prefs.CmdBarIsActive == IsActive)
	{
		switch (Swap16(DequedEvent->eType))
		{
		case keyDownEvent:
			switch (Swap16(DequedEvent->data.keyDown.chr))
			{
			case vchrMenu:
			case vchrKeyboard:
			case vchrFind:
			case vchrCalc:
			case vchrKeyboardAlpha:
			case vchrKeyboardNumeric:
			case vchrTsm1:
			case vchrTsm2:
			case vchrTsm3:
			case vchrTsm4:
			case vchrBrightness:
			case vchrContrast:
				Deactivate();
				break;

			case chrSpace:	//chrSpace 0x0020
				DequedEvent->data.keyDown.chr = 0x1801;	//vchrTsm1 0x0118 Henkan
				DequedEvent->data.keyDown.modifiers = 0x0800;
				Deactivate();
				break;

			case vchrLaunch:	//vchrLaunch 0x0108
			case chrLineFeed:	//chrLineFeed 0x000a
				DequedEvent->data.keyDown.chr = 0x1a01;	//vchrTsm3 0x011a HiraKata
				DequedEvent->data.keyDown.modifiers = 0x0800;
				Deactivate();
				break;

			case vchrCommand:	//vchrCommand 0x0106
				DequedEvent->data.keyDown.chr = 0x1b01;	//vchrTsm4 0x011b Nichiei
				DequedEvent->data.keyDown.modifiers = 0x0800;
				Deactivate();
				break;

			case chrBackspace:	//chrBackspace 0x0008
				DequedEvent->data.keyDown.chr = 0x0801;	//vchrLaunch 0x0108 Launch
				DequedEvent->data.keyDown.modifiers = 0x0800;
				Deactivate();
				break;
			}
			break;

		case menuEvent:
			Deactivate();
			break;
		}
	}
} 

#pragma mark ----------

static void SetFEPTglPreference(FEPTglPreferenceType *prefs)
{
	PrefSetAppPreferences (appFileCreator, appPrefID, appPrefVersionNum, 
		prefs, sizeof (FEPTglPreferenceType), false);
}

static void GetFEPTglPreference(FEPTglPreferenceType *prefs)
{
	UInt16 prefsSize;

	prefsSize = sizeof(FEPTglPreferenceType);
	if (PrefGetAppPreferences(appFileCreator, appPrefID, prefs, &prefsSize, false) == 
		noPreferenceFound)
		{
			prefs->On         = IsOff;
			prefs->Off        = IsOn;
			prefs->AutoEnbleF = IsOff;
			prefs->BlcAtShftF = IsOff;
		}
	prefs->CmdBarIsActive = IsDeactive;
	prefs->UnUsed = 0;
	SetFEPTglPreference(prefs);
}

#pragma mark ----------

static void Disable(void)
{
	UInt16 cardNo;
	LocalID dbID;
	SysCurAppDatabase (&cardNo, &dbID);

	SysNotifyUnregister(cardNo, dbID, sysNotifyMenuCmdBarOpenEvent, sysNotifyNormalPriority);
	SysNotifyUnregister(cardNo, dbID, sysNotifyEventDequeuedEvent, sysNotifyNormalPriority);

	FtrUnregister (appFileCreator, appLocalID);
	FtrUnregister (appFileCreator, appStatus );
}

static void Enable(void)
{
	UInt16 cardNo;
	LocalID dbID;
	SysCurAppDatabase (&cardNo, &dbID);

	SysNotifyRegister (cardNo, dbID, sysNotifyMenuCmdBarOpenEvent, NULL, sysNotifyNormalPriority, NULL);

	FEPTglPreferenceType prefs;
		GetFEPTglPreference(&prefs);
	FtrSet (appFileCreator, appLocalID, dbID );
	FtrSet (appFileCreator, appStatus , *(UInt32*)&prefs);

	if (prefs.BlcAtShftF == IsOn)
		SysNotifyRegister (cardNo, dbID, sysNotifyEventDequeuedEvent, NULL, sysNotifyNormalPriority, NULL);
}

#pragma mark ----------

static void GetCtrlValue(FormPtr frmP)
{
	FEPTglPreferenceType prefs;

	if ( FrmGetControlValue (frmP, FrmGetObjectIndex (frmP, MainONPushButton)) == true)
		prefs.On = IsOn; else prefs.On = IsOff;
	if ( FrmGetControlValue (frmP, FrmGetObjectIndex (frmP, MainOFFPushButton)) == true)
		prefs.Off = IsOn; else prefs.Off = IsOff;
	if ( FrmGetControlValue (frmP, FrmGetObjectIndex (frmP, MainEnblRstCheckbox)) == true)
		prefs.AutoEnbleF = IsOn; else prefs.AutoEnbleF = IsOff;
	if ( FrmGetControlValue (frmP, FrmGetObjectIndex (frmP, MainBlcAtShftCheckbox)) == true)
		prefs.BlcAtShftF = IsOn; else prefs.BlcAtShftF = IsOff;

	SetFEPTglPreference(&prefs);
	
	if (prefs.On == IsOn)
	{
		Enable();
	}
	else
	{
		Disable();
	}
}

static void SetCtrlValue(FormPtr frmP)
{
	FEPTglPreferenceType prefs;
	GetFEPTglPreference(&prefs);

	FrmSetControlValue (frmP, FrmGetObjectIndex (frmP, MainONPushButton     ), prefs.On        );
	FrmSetControlValue (frmP, FrmGetObjectIndex (frmP, MainOFFPushButton    ), prefs.Off       );
	FrmSetControlValue (frmP, FrmGetObjectIndex (frmP, MainEnblRstCheckbox  ), prefs.AutoEnbleF);
	FrmSetControlValue (frmP, FrmGetObjectIndex (frmP, MainBlcAtShftCheckbox), prefs.BlcAtShftF);
}

#pragma mark ----------

static UInt32 StarterPalmMain(UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
{
//	Err error;

	switch (cmd)
		{
		case sysAppLaunchCmdNotify:
			SysNotifyParamType* NtfyPrm = (SysNotifyParamType*)cmdPBP;
			switch (NtfyPrm->notifyType)
				{
				case sysNotifyEventDequeuedEvent:
					EventPtr DequedEvent;
					DequedEvent = (EventType*)NtfyPrm->notifyDetailsP;
					NtfyHandle (DequedEvent);
					break;

				case sysNotifyMenuCmdBarOpenEvent:
					Activate();
					break;
				}
			break;

		case sysAppLaunchCmdAlarmTriggered:
			Deactivate ();
			break;

		case sysAppLaunchCmdNormalLaunch:
			FormPtr frmP = FrmInitForm (MainForm);		//Initialize dialogue form
			SetCtrlValue(frmP);				 			//Set control value
			if (FrmDoDialog (frmP) == MainDoneButton)	//Draw dialogue form
				GetCtrlValue(frmP);						//Get control value
			break;

		case sysAppLaunchCmdSystemReset:
			FEPTglPreferenceType prefs;
			GetFEPTglPreference(&prefs);
			if (prefs.On == IsOn)
			{
				if (prefs.AutoEnbleF == IsOn)
					Enable();						//activate
				else
				{
					prefs.On  = IsOff;
					prefs.Off = IsOn;
					SetFEPTglPreference(&prefs);
				}
			}
			break;

		default:
			break;
		}
	return errNone;
}

UInt32 PilotMain( UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
{
	return StarterPalmMain(cmd, cmdPBP, launchFlags);
}
